-- fundraw.ex
-- Author: Normand M. Blais
-- Date: December 1998
-- A simple drawing program using the turtle graphic library.
-- Commands:
--                Key            Command
--                UP_ARROW       move NORTH
--                DOWN_ARROW     move SOUTH
--                LEFT_ARROW     move WEST
--                RIGHT_ARROW    move EAST
--                PAGE_UP        move NORTH-EAST
--                PAGE_DOWN      move SOUTH-EAST
--                END            move SOUTH-WEST
--                DELETE         move NORTH-WEST
--                HOME           move to the center of window 
--                SPACE_BAR      toggle between active color and window
--                               color
--                F4             decrement color number (active color)
--                F5             increment color number (active color)
--                F6             Show/hide turtle cursor
--                F8             Clear window drawing
--                F10/q/ESC      Quit program
--
include get.e
include keymap.e
include turtle.e
 
constant WIDTH = 540, HEIGHT = 350 
constant LX = 55, HX = 585, LY = 70, HY = 412
constant BKCOLOR = RED

integer active_color

procedure pause()
    integer k
    k = wait_key()
end procedure     

procedure show_active_color()
    polygon(BLACK,1,{{MaxX-40,MaxY-10},{MaxX-10,MaxY-10},
                     {MaxX-10,MaxY-30},{MaxX-40,MaxY-30}})
    polygon(active_color,1,{{MaxX-38,MaxY-12},{MaxX-12,MaxY-12},
                            {MaxX-12,MaxY-28},{MaxX-38,MaxY-28}})
end procedure

procedure toggle_colors(integer n)
    if n > 0 then
       active_color = active_color + 1
       if active_color >= maxcolor then
           active_color = BLACK    
       end if
    elsif n < 0 then
       active_color = active_color - 1
       if active_color < 0 then
           active_color = maxcolor - 1
       end if
    end if
end procedure

procedure set_window(integer xaxis, integer yaxis, integer width, 
                     integer height)
   integer xlimit, ylimit, left,right,top,bottom

   xlimit = MaxX - 3
   ylimit = MaxY - 3
   left = xaxis - floor(width / 2)
   while left < 3 do
       left = left + 1
   end while
   right = left + width 
   while right > xlimit do
       right = right - 1
  end while
   top = yaxis - floor(height / 2)
   while top < 3 do
       top = top + 1
   end while
   bottom = top + height
   while bottom > ylimit do
       bottom = bottom - 1
   end while 
   polygon(WHITE,1,{{left,top},{right,top},{right,bottom},
                    {left,bottom}})
end procedure     

procedure init_fundraw()
    init_turtle()
    polygon(BKCOLOR,1,{{0,0},{MaxX-1,0},{MaxX-1,MaxY-1},{0,MaxY-1}})
    set_window(MaxX/2,MaxY/2, WIDTH, HEIGHT)
    turtle_window(MaxX/2,MaxY/2, WIDTH, HEIGHT)
    active_color = BLACK
    set_pen_color(active_color)
    show_active_color()
    visible = 0
end procedure

procedure  show_loc()
    position(3,3)
    printf(1,"%3d", xpos)
    position(3, 8)
    printf(1,"%3d", ypos)
end procedure

procedure main_loop()
    integer k,sav_color
    sav_color = active_color
    show_loc()
    while 1 do
        k = get_key()
        if k = ESC or k = 'q' or k = F10 then
           exit 
        elsif k = UP_ARROW then
           set_heading(NORTH)  -- 0 degree
           if ypos > LY then
               forward(1) 
               show_loc()
           end if
        elsif k = PGUP then
           set_heading(45)    -- 45 degrees
           if xpos < HX and ypos > LY then
               forward(1.5) 
               show_loc()
           end if
        elsif k = RIGHT_ARROW then
           set_heading(EAST)   -- 90 degrees
           if xpos < HX then
               forward(1) 
               show_loc()
           end if
        elsif k = PGDOWN then
           set_heading(115)    -- 135 degrees
           if xpos < HX and ypos < HY then
               forward(2) 
               show_loc()
           end if
        elsif k = DOWN_ARROW then
           set_heading(SOUTH)  -- 180 degrees
           if ypos < HY then
               forward(1) 
               show_loc()
           end if
        elsif k = END then
           set_heading(225)    -- 225 degrees
           if xpos > LX and ypos < HY then
               forward(1) 
               show_loc()
           end if
        elsif k = LEFT_ARROW then
           set_heading(WEST)   -- 270 degrees
           if xpos > LX then
               forward(1) 
               show_loc()
           end if
        elsif k = DELETE then
           set_heading(345)    -- 315 degrees
           if xpos > LX and ypos > LY then
               forward(2) 
               show_loc()
           end if
        elsif k = HOME then
           home()
           show_loc()
        elsif k = F8 then
           home()
           hide_turtle()
           clear_turtle_screen(WHITE)
           show_loc()
        elsif k = F4 then
           toggle_colors(-1)
           set_pen_color(active_color)
           show_active_color()
        elsif k = F5 then
           toggle_colors(1)
           set_pen_color(active_color)
           show_active_color()
        elsif k = F6 then
           if not visible then
               show_turtle()
           else
               hide_turtle()    
           end if
        elsif k = SPACE then
           if active_color = WHITE then
               active_color = sav_color
           else
               sav_color = active_color    
               active_color = WHITE
           end if
           set_pen_color(active_color)
           show_active_color()
        end if
    end while
end procedure

procedure end_fundraw()
    end_turtle()
end procedure

procedure fundraw()
    init_fundraw()
    main_loop()
    end_fundraw()    
end procedure

fundraw()

