-- turtle.e
--
-- Translate from C to Euphoria by Normand M. Blais
-- date: December 1998
-- Reference:
--      "Graphics Programming in Turbo C 2.0" by Ben Ezzel.
-- 
------------------------------<Include>---------------------------------------
include graphics.e
include image.e

------------------------------<Global>---------------------------------------
global constant NORTH=0,EAST=90,SOUTH=180,WEST=270
global integer maxcolor, xpos, ypos
global integer MaxX, MaxY
global integer visible

------------------------------<Private>--------------------------------------
constant PI=4.0*arctan(1.0),
         FALSE=0,TRUE=1

sequence turtle, sav_img

atom timeout

integer direction,draw,pencolor,wrapoff,
        xhome,yhome,
        left,right,top,bottom

------------------------------<delay>---------------------------------------
global
procedure delay(integer m)
--
-- procedure delay(integer d)
--
-- Pauses the action for m milliseconds
--
    atom tm                       
    tm = time() + m / 1000
    while time() < tm do end while
end procedure

------------------------------<turtle_delay>---------------------------------
global
procedure turtle_delay(integer delay)
--
-- procedure turtle_delay(integer delay)
--
-- The turtle delay allowed to specify a delay in milliseconds
-- between steps (pixel movements).
-- Initially, the delay is 0.
--
   timeout = delay  -- in milliseconds
end procedure

------------------------------<create_turtle>--------------------------------
global
procedure create_turtle()
--
-- procedure create_turtle()
--
-- Draws a simple cursor shape and save its image for later use.
--
   sav_img = save_image({1,1},{5,5})
   for i=1 to 5 do
       pixel(maxcolor-1,{i,3})
   end for
   for i=1 to 5 do
       pixel(maxcolor-1,{3,i})
   end for
   turtle = save_image({1,1},{5,5})
   display_image({1,1},sav_img)
end procedure    

------------------------------<draw_turtle>----------------------------------
global
procedure draw_turtle()
--
-- procedure draw_turtle()
--
-- Uses to draw the turtle cursor on the screen at current 
-- position (xpos,ypos).
--
   integer x1,y1,x2,y2
   x1 = xpos-2  x2 = xpos+2
   if x1 < 0 then
      x1 = 0  x2 = x2 + 1 
   end if
   y1 = ypos-2  y2 = ypos+2
   if y1 < 0 then
      y1 = 0 y2 = y2 + 1 
   end if
   sav_img = save_image({x1,y1},{x2,y2})
   if visible then
       display_image({x1,y1},xor_bits(sav_img,turtle))
   end if
end procedure

------------------------------<rsin>-----------------------------------------
global
function rsin(integer degrees)
--
-- function rsin(integer degrees)
--
-- Returns the sin value of an angle in degree.
--
   return sin(PI*degrees/180)
end function

------------------------------<rcos>-----------------------------------------
global
function rcos(integer degrees)
--
-- function rcos(integer degrees)
--
-- Returns the cosin value of an angle in degree.
--
   return cos(PI*degrees/180)
end function

------------------------------<correct_direction>----------------------------
global
procedure correct_direction()
--
-- procedure correct_direction()
--
-- keep the heading angle in the range of 0-360 degrees.
--
   while direction > 360 do
       direction = direction - 360    
   end while
   while direction < 0 do
       direction = direction + 360    
   end while
end procedure

------------------------------<turtle_where>---------------------------------
global
function turtle_where()
--
-- function turtle_where()
--
-- Returns TRUE if the current turtle position is within the
-- window limits. FALSE otherwise.
--
   if xpos >= left and xpos <= right and 
      ypos >= top and ypos <= bottom then
      return TRUE 
   else
      return FALSE
   end if
end function

------------------------------<home>-----------------------------------------
global
procedure home()
--
-- procedure home()
--
-- Moves the cursor at the center of the window (0,0) without 
-- drawing a line in the process. If a delay is in effect, the
-- cursor move to the center of the screen, but no drawing
-- occurs until the time is up.
--
   if visible then
       draw_turtle()
   end if
   xpos = xhome
   ypos = yhome
   if timeout then
       delay(timeout)        
   end if
   if visible then
       draw_turtle()
   end if
end procedure

------------------------------<clear_turtle_screen>--------------------------
global
procedure clear_turtle_screen(integer color)
--
-- procedure clear_turtle_screen(integer color)
--
-- Erases the turtle window using the color argument.
-- This is more flexible than using a single default color.
--
   polygon(color,1,{{left,top},{right,top},
                   {right,bottom},{left,bottom}})
end procedure


------------------------------<abs>------------------------------------------
global
function abs(integer n)
--
-- function abs(integer n)
--
-- Returns the absolute value of an integer n.
-- 
   if n < 0 then
      return -n 
   end if      
   return n
end function

------------------------------<pen_down>--------------------------------------
global
procedure pen_down()
--
-- procedure pen_down()
--
-- Sets the common variable draw to TRUE so the pixels are
-- lighted on the screen.
--
   draw = TRUE
end procedure

------------------------------<pen_up>---------------------------------------
global
procedure pen_up()
--
-- procedure pen_up()
--
-- Sets the common variable draw to FALSE so no pixel is
-- lighted on the screen.
--
   draw = FALSE
end procedure

------------------------------<show_turtle>----------------------------------
global
procedure show_turtle()
--
-- procedure show_turtle()
--
-- Enables the turtle cursor.
-- Initialially, the cursor is disabled.
--
   visible = TRUE
   if visible then
       draw_turtle()
   end if
   if timeout then
       delay(timeout)        
   end if
end procedure    

------------------------------<step_turtle>----------------------------------
global
procedure step_turtle(integer xstep, integer ystep)
--
-- procedure step_turtle(integer xstep, integer ystep)
--
-- Lights up a new pixel at new current location (xstep,ystep).
-- If visible is TRUE, the turtle cursor is also drawn on the
-- screen. Not otherwise. If there is a delay in effect, pause
-- until the time is up.
--
    if visible then 
        draw_turtle()
    end if
    xpos = xstep
    ypos = ystep
    if not wrapoff then
        if xpos < left then
            xpos = xpos + right
        elsif xpos > right then
            xpos = xpos - right 
        end if
        if ypos < top then
            ypos = ypos + bottom
        elsif ypos > bottom then
            ypos = ypos - bottom
        end if
    end if                     
    if draw = TRUE and turtle_where() then
        pixel(pencolor,{xpos,ypos}) 
        if visible then 
            draw_turtle()
        end if
        if timeout and turtle_where() then
           delay(timeout)    
        end if
    end if
end procedure


------------------------------<drawstr>--------------------------------------
global
procedure drawstr(integer scale, sequence str)
--
-- procedure drawstr(integer scale, sequence str)
--
-- An alternative to the routine forward(). But unlike
-- forward the movements are executed in absolute pixel
-- steps and they limited to eight directions: 0,45,90,
-- 135,180,225,270,315,360. The routine accepts two ar-
-- guments: a scale multiplier and a sequence of charac-
-- ters describing the turtle movements that create a 
-- figure, logo or other drawing. Scale is the unit of
-- distance. The basic commands are:
--         'm' --> no movement, pen down
--         'p' --> no movement, pen up
--         'a' --> move SOUTH      (180)
--         'b' --> move SOUTH-EAST (135)
--         'c' --> move EAST       (90)
--         'd' --> move NORTH-EAST (45)
--         'e' --> move NORTH      (0)
--         'f' --> move NORTH-WEST (315)
--         'g' --> move WEST       (270)
--         'h' --> move SOUTH-WEST (225)
--         '2' --> mult distance by 1
--         '3' --> mult distance by 2
--          ..........................
--         '9' --> mult distance by 8
--         '0' --> mult distance by 9
--
   integer distance,x,y
   for j=1 to length(str) do
       distance = scale
       if str[j] = 'a' then
           x = 0 y = -1
       elsif str[j] = 'b' then
           x = 1 y = -1    
       elsif str[j] = 'c' then
           x = 1 y = 0    
       elsif str[j] = 'd' then
           x = 1 y = 1
       elsif str[j] = 'e' then
           x = 0 y = 1    
       elsif str[j] = 'f' then
           x = -1 y = 1    
       elsif str[j] = 'g' then
           x = -1 y = 0    
       elsif str[j] = 'h' then
           x = -1 y = -1
       elsif str[j] = 'm' then
           x = 0 y = 0
           pen_down()    
       elsif str[j] = 'p' then
           x = 0 y = 0    
           pen_up()    
       elsif str[j] = '2' then
           -- distance = scale * 1
       elsif str[j] = '3' then
           distance = scale * 2
       elsif str[j] = '4' then
           distance = scale * 3
       elsif str[j] = '5' then
           distance = scale * 4
       elsif str[j] = '6' then
           distance = scale * 5
       elsif str[j] = '7' then
           distance = scale * 6
       elsif str[j] = '8' then
           distance = scale * 7
       elsif str[j] = '9' then
           distance = scale * 8
       elsif str[j] = '0' then
           distance = scale * 9
       else 
           x = 0 y = 0    
       end if
       for i=1 to distance do
           step_turtle(xpos+x,ypos+y)    
       end for
   end for        
end procedure

------------------------------<forward>--------------------------------------
global
procedure forward(atom distance)
--
-- procedure forward(atom distance)
--
-- Moves the turtle cursor the distance specified from
-- the current position along the current heading.
-- The movement unit is one pixel and if the pen is down,
-- a line is draw in the process. If the pen is up, no
-- line is drawn. If the distance is negative, the move-
-- ment is execute in the direction opposite to the cur-
-- rent heading. If a delay is in effect, a pause will 
-- occur in between steps of the cursor.
-- Uses: rcos() and rsin() functions to calculate the
--       appropriate x-axis and y-axis distances.
--       step_turtle() to accomplish the actual movment.
--
   integer xdistance, ydistance, xorg, yorg
   atom slope
   xorg = xpos yorg = ypos
   xdistance = floor(rsin(direction) * distance)
   ydistance = -floor(rcos(direction) * distance)
   if abs(xdistance) > abs(ydistance) then
       slope = ydistance / xdistance
       if xdistance > 0 then
           for i=1 to xdistance do
               step_turtle(xorg+i,yorg+floor(i*slope))
           end for
       else
           for i=-1 to xdistance by -1 do
               step_turtle(xorg+i,yorg+floor(i*slope))
           end for
       end if
   else
       slope = xdistance / ydistance
       if ydistance > 0 then
           for i=1 to ydistance do
               step_turtle(xorg+floor(i*slope),yorg+i)
           end for
       else
           for i=-1 to ydistance by -1 do
               step_turtle(xorg+floor(i*slope),yorg+i)
           end for
       end if
   end if
end procedure

------------------------------<back>-----------------------------------------
global
procedure back(integer distance)
--
-- procedure back(integer distance)
--
-- Acts like forward but in reverse direction.
--
   return forward(-1 * distance)
end procedure

------------------------------<heading>--------------------------------------
global
function heading()
--
-- function heading()
--
-- Returns the current direction of the cursor on the screen.
-- The return value is an integer.
--
  return direction
end function

------------------------------<hide_turtle>----------------------------------
global
procedure hide_turtle()
--
-- procedure hide_turtle()
--
-- Disables the turtle cursor.
-- Initialially, the cursor is disabled.
--
   if visible then
       draw_turtle()
   end if
   visible = FALSE
   if timeout then
       delay(timeout)        
   end if
end procedure

------------------------------<no_wrap>--------------------------------------
global
procedure no_wrap()
--
-- procedure no_wrap()
--
-- Turns on the wrapping option which allowed the movement of the
-- turtle to pass the limit of the window and to reenter at the
-- opposite border.
--
   wrapoff = TRUE
end procedure


------------------------------<set_heading>----------------------------------
global
procedure set_heading(integer degrees)
--
-- procedure set_heading(integer degrees)
--
-- Sets the current heading to an absolute value.
-- Calls the procedure correct_direction() to keep
-- the values in the range of 0 - 360 degrees 
--
   direction = degrees
   correct_direction()
end procedure  

------------------------------<set_pen_color>--------------------------------
global
procedure set_pen_color(integer color)
--
-- procedure set_pen_color(integer color)
--
-- Sets the active color for drawing.
--
    if color < 0 then
        color = 0  
    end if
    if color > maxcolor then
        color = maxcolor    
    end if              
    pencolor = color
end procedure

------------------------------<set_position>---------------------------------
global
procedure set_position(integer xaxis, integer yaxis)
--
-- procedure set_position(integer xaxis, integer yaxis)
--
-- Moves the turtle cursor immediately to the turtle coordinates
-- specified. No line is drawn. If a delay is in effect, no drawing
-- is possible until the time is up.
--
   if visible then
       draw_turtle()
   end if
   xpos = xhome + xaxis
   ypos = yhome - yaxis
   if timeout then
       delay(timeout)        
   end if
   if visible then
       draw_turtle()
   end if
end procedure       

------------------------------<turn_left>------------------------------------
global
procedure turn_left(integer degrees)
--
-- procedure turn_left(integer degrees)
--
-- Decreases the current heading by degrees (conterclockwise)
--
   direction = direction + degrees
   correct_direction()
end procedure

------------------------------<turn_right>-----------------------------------
global
procedure turn_right(integer degrees)
--
-- procedure turn_right(integer degrees)
--
--
-- Increases the current heading by degrees (clockwise)
--
   direction = direction - degrees
   correct_direction()
end procedure             


------------------------------<turtle_window>--------------------------------
global
procedure turtle_window(integer xaxis, integer yaxis, 
                        integer width, integer height)
--                        
-- procedure turtle_window(integer xaxis, integer yaxis, 
--                         integer width, integer height)
--
-- Defines an area of the screen as the active turtle window.
-- The first two argument xaxis and yaxis are the pixel location of the
-- center of the window in screen coordinate (x:0->639, y:0->479).
-- The arguments width and height are the dimensions of the window in
-- number of pixels. Once the window is defined, the center of the window
-- becomes the turtle (cursor) center and its location in turtle coordinates
-- is (0,0). The window is then divided in four quadrants: 
--                                   
--                                  |
--                           (-,+)  |   (+,+)           
--                                  |
--                        ----------|------------
--                           (-,-)  |   (+,-)
--                                  |
--                                  | 
--
--  The following global variables are set:
--      xhome = xpos = 0 -- x_center
--      yhome = ypos = 0 -- y_center
--
--  Note: A margin is provided to prevent the turtle cursor to be drawn
--        outside the limits of the window.
-- 
   integer xlimit, ylimit
   xlimit = MaxX - 3
   ylimit = MaxY - 3
   left = xaxis - floor(width / 2)
   while left < 3 do
       left = left + 1
   end while
   right = left + width 
   while right > xlimit do
       right = right - 1
  end while
   top = yaxis - floor(height / 2)
   while top < 3 do
       top = top + 1
   end while
   bottom = top + height
   while bottom > ylimit do
       bottom = bottom - 1
   end while 
   xhome = xaxis
   yhome = yaxis
   xpos = xhome
   ypos = yhome 
end procedure     


------------------------------<wrap_>----------------------------------------
global
procedure wrap_()         
--
-- procedure wrap_()         
--
-- Turns off the wrapping option which allowed the movement of the
-- turtle to pass the limit of the window and to reenter at the
-- opposite border.
-- Note: Euphoria has a routine called wrap which forces us to use
--       a different name for the present procedure.
--
   wrapoff = FALSE
end procedure

------------------------------<xcor>-----------------------------------------
global
function xcor()
--
-- function xcor()
--
-- Returns the x-axis of the current turtle cursor position.
--
   return xpos - xhome
end function

------------------------------<ycor>-----------------------------------------
global
function ycor()
--
-- function ycor()
--
-- Returns the y-axis of the current turtle cursor position.
--
   return -1 * (ypos - yhome)
end function


------------------------------<init_turtle>----------------------------------
global
procedure init_turtle()
--
-- procedure init_turtle()
--
-- Initializes the turtle graphic system:
--    init the graphic mode (18 = 640 x 480 16 colors)
--    create a turtle cursor and save its image
--    init the turtle window
--    update global variables:
--          MaxX = maximum number of pixels in x direction (640)
--          MaxY = maximum number of pixels in y direction (480)
--          maxcolor = maximum number of colors (16)
--    set the pen color to maxcolor - 1
--    set no wrap around option
--    set the pen down ready to draw 
--    set the direction to 0 degree (NORTH)
--    set the turtle delay to 0 (maximum speed)
--    set the turtle cursor to invisible
-- note: The aspect ration for mode 18 is 1 and is not taking into 
--       account.
--
   sequence vc
   if graphics_mode(18) then
   end if
   vc = video_config()
   maxcolor = vc[VC_NCOLORS]
   create_turtle()
   MaxX = vc[VC_XPIXELS]
   MaxY = vc[VC_YPIXELS] 
   turtle_window(MaxX/2,MaxY/2,MaxX,MaxY)
   set_pen_color(maxcolor)
   no_wrap()
   pen_down()
   set_heading(0)
   turtle_delay(0)
   visible = FALSE
end procedure

------------------------------<end_turtle>-----------------------------------
global
procedure end_turtle()
--
-- procedure end_turtle()
--                       
-- Reset to text mode.
--
   if graphics_mode(-1) then
   end if
end procedure

-----------------------------------------------------------------------------

